﻿<#
.Synopsis
   Runs many components, that check the status of items used by EL. 
.DESCRIPTION
   Invoke-ELHC looks at an Evidence Library 4.0+ installation,
   and determines the relative health via a number of factors,
   including ACL permissions, # of events by category, and health
   of the storage shares used for video retention.
   
.EXAMPLE
  PS c:\> Invoke-ELHC
  Runs Healthcheck and reports to OMS

.EXAMPLE
  PS c:\> Invoke-ELHC -Verbose
  This provides verbose output of config values, components, and current status

.NOTES
Author: Dave Wixon
Date: 2/15/18
Version: 1.3082

#>
function Invoke-ELHC
{
    ##Requires -RunAsAdministrator
    #Requires -Version 5.0
    
    [CmdletBinding(SupportsShouldProcess=$true, 
                  PositionalBinding=$false,
                  ConfirmImpact='Medium')]
    [Alias()]
    [OutputType([PSOBJECT])]

    Param (
        [Parameter(Mandatory=$false, Position=0)]
        [ValidateScript({ Test-Path -Path $_ -PathType File })]
        [string]$ConfigPath = "$PSScriptRoot\ELHC.config",

        [Parameter(Mandatory=$false)]
        [switch]$NoReport = $false
    )

    $NewVersion = (Find-Module -Repository ELHCRepo -Name ELHealthCheck).Version.ToString()
    $CurrentVersion = (Get-InstalledModule -Name ELHealthCheck).Version.ToString()

    #if we are updating, copy over the config and restart ELHC
    if ($NewVersion -gt $CurrentVersion) {
        $OldPath = (get-installedmodule ELHealthcheck -RequiredVersion $CurrentVersion).InstalledLocation
        Find-Module -Repository ELHCRepo -Name ELHealthcheck | Install-Module -Force
		$NewPath = (get-installedmodule ELHealthcheck -RequiredVersion $NewVersion).InstalledLocation
       
		Copy-item -path "$OldPath\ELHC.config" -destination "$NewPath\ELHC.config" -Force
        Start-Process powershell.exe -verb runas -ArgumentList "-NoProfile -command `"& {import-module -name ELHealthcheck -version $NewVersion -force; uninstall-module -name ELHealthcheck -requiredversion $CurrentVersion -force; invoke-elhc -verbose}`""
        exit
    }

    $DateTime = Get-Date -Format 'yyyy-MM-ddThh.mm.ss'
    Start-Transcript -Path "$PSScriptRoot\PastOutput\ELHC_Transcript_$DateTime.txt" -InformationAction Continue -ErrorAction SilentlyContinue

    #Load the config and start up ELHC
    Update-ELHCConfig -ConfigPath $ConfigPath
    $LoadConfig = . {
        Write-Output "`nImporting configuration files...."
        if (Test-Path -Path $ConfigPath) {
            [xml]$ELHCConfig = Get-Content -Path $ConfigPath

        }
        else { 
            Write-Warning -Message "Did not find environmental info.  Get this from a PM"
            Return
        }
    };

    New-ELHC -version $CurrentVersion -timeout 3

    #run sub-functions, storing result hashtables into an array of hashtables
    $InstanceGUID = (New-Guid).Guid
    $RunComponents = . {
        
        #import/dot source the components
        $ImportComponents = . {
            $TotalComponents = 0
            $AvduRows = Get-QueryResult -query "Select * from storage.AvduStorage WHERE SubstationID = '$($ELHCConfig.root.SubstationID)'" -ConnectionString $ELHCConfig.root.ConnectionString 

            Write-Verbose "Importing components...`n"
            $StandaloneComponents = @()
            foreach ($Component in $ELHCConfig.root.components.standalone.component) {
                Write-Verbose "------$($Component.name) : $($Component.enabled)" 
                if ($Component.enabled -eq $true) {
                    $StandaloneComponents += $Component.name
                    $TotalComponents++
                } 
            }
            $AvduComponents = @()
            foreach ($Component in $ELHCConfig.root.components.avdurow.component) {
                Write-Verbose "------$($Component.name) : $($Component.enabled)"
                if ($Component.enabled -eq $true) {
                    $AvduComponents += $Component.name
                    $TotalComponents += $AvduRows.length
                } 
            }
        }
        
        #these components run standalone, not needing any external data to produce output
        $RunStandaloneComponents = . {
            Write-Verbose ""
            Write-Verbose "Running standalone components..."
            
            #somewhere to put the hashtables returned by components
            $ComponentData = @()
            $ComponentCount = 0
            foreach ($Component in $StandaloneComponents) {
                #update the host
                Write-Verbose "------Processing... $($Component)"
                Write-Progress -Activity "Processing standalone components..." -Status $Component -PercentComplete ($ComponentCount*100/$TotalComponents)
                $ComponentCount++       
                $BaseAvduHash = @{
                    'DateGenerated' = get-date -Format 'yyyy-MM-ddTHH:mm:ss.fffzzz'
                    'DepartmentID' = $ELHCConfig.root.DepartmentID
                    'DepartmentName' = $ELHCConfig.root.DepartmentName
                    'InstanceGUID' = $InstanceGUID
                }
                    
                $ScriptBlock = {& $args[0] -ELHCConfig $args[1]}
                $JobID = (Start-Job -ScriptBlock $ScriptBlock -ArgumentList $Component,$ELHCConfig).Id
                
                Wait-Job -Id $JobID -Timeout $ELHCConfig.root.ComponentTimeout
                if ((Get-Job -Id $JobID).State -eq 'Completed') {
                    $JobResult = Receive-Job -Id $JobID
                }
                else { Write-Warning "---------$Component timed out after $($ELHCConfig.root.ComponentTimeout) seconds."}

                $BaseAvduHash += $JobResult
                $ComponentData += $BaseAvduHash
            }
        }
        
        
        #these components require a single AvduStorage Row, and produce unique output per row
        $RunAvduComponents = . {
            foreach ($AvduRow in $AvduRows) {
                Write-Verbose ""
                Write-Verbose "Running components on... $($AvduRow.Path)"
                foreach ($Component in $AvduComponents) {
                    #update the host
                    Write-Verbose "------Processing... $($Component)"
                    Write-Progress -Activity "Processing share components..." -Status $Component -PercentComplete (($ComponentCount+1)*100/$TotalComponents)
                    $ComponentCount++        
                    #boilerplate info added to each component log output
                    $BaseAvduHash = @{
                        'AvduStorageName' = $AvduRow.Name
                        'AvduStoragePath' = $AvduRow.Path
                        'AvduStorageType' = $AvduRow.Type
                        'AvduStorageQuota' = $AvduRow.QuotaLimit
                        'DateGenerated' = get-date -Format 'yyyy-MM-ddTHH:mm:ss.fffzzz'
                        'DepartmentID' = $ELHCConfig.root.DepartmentID
                        'DepartmentName' = $ELHCConfig.root.DepartmentName
                        'InstanceGUID' = $InstanceGUID
                    }
                      
                    $ScriptBlock = {& $args[0] -ELHCConfig $args[1] -AvduRow $args[2]}
                    $JobID = (Start-Job -ScriptBlock $ScriptBlock -ArgumentList $Component,$ELHCConfig,$AvduRow).Id
                
                    Wait-Job -Id $JobID -Timeout $ELHCConfig.root.ComponentTimeout
                    if ((Get-Job -Id $JobID).State -eq 'Completed') {
                        $JobResult = Receive-Job -Id $JobID
                    }
                    else { Write-Warning "---------$Component timed out after $($ELHCConfig.root.ComponentTimeout) seconds."}

                    $BaseAvduHash += $JobResult
                    $ComponentData += $BaseAvduHash

                }
            }
        }
    };

    #Iterate through our component output array, posting each to OMS w/unique LogType
    $ConvertAndPOST = . {
        $LogPath = "$PSScriptRoot\PastOutput\ELHC_$DateTime"
        New-Item -Path "$LogPath.json" -Force

        foreach ($Log in $ComponentData) {
            if ($Log.LogType -eq 'Environment') {
                $ContainsEnvironment = $true
            }
        }
        
        $SkipPOST = {
            foreach ($Log in $ComponentData) {
                $Log = ConvertTo-Json $Log -Depth 3
                $Log | Out-File -FilePath "$LogPath.json" -Append
            }
        }

        if ($NoReport) {
            Write-Verbose "-NoReport flag -- results will not upload to OMS"
            .$SkipPOST
        } elseif (!$ContainsEnvironment) {
            Write-Verbose "Environment logs are not present.  Results will not upload to OMS"
            .$SkipPOST
        }

        else {
            $c = 0
            foreach ($Log in $ComponentData) {
                Write-Progress -activity "Uploading.... $LogType logs" -PercentComplete (($c*100)/$ComponentData.length)
                $LogType = $Log.LogType
                $Log = ConvertTo-Json $Log -Depth 3
                $Log | Out-File -FilePath "$LogPath.json" -Append
                try {
                    $VerbosePreference = 'silentlycontinue'
                    $POST = Push-OMSData `
                        -json $Log `
                        -CustomerID $ELHCConfig.root.CustomerID `
                        -SharedKey $ELHCConfig.root.SharedKey `
                        -LogType $LogType `
                        -DateStamp (Get-Date)
                    $VerbosePreference = 'continue'
                    $c++
                }
                catch {
                    Write-Warning "Failed to upload $LogType logs... $_"
                    $failure = $true
                }
            }
            if ($failure) { Write-Verbose "`nOne or more logs failed to upload..." }
            else { Write-Verbose "`nCongratulations, your data has been uploaded successfully." }
        }

        foreach ($Log in $ComponentData) {
            if ($Log.AlertBool -eq $true) {
                Write-Warning "$($Log.Logtype) raised an alert"
                Start-Sleep -Milliseconds 500
                $PassedELHC = $false
                $Log | Out-Host
            }
        }

        Write-Verbose "Component output located at `'$LogPath.json`'"
    }

    Stop-Transcript

}

  