﻿function Set-ELHCConfig
{
    #Requires -RunasAdministrator
    [CmdletBinding(SupportsShouldProcess=$true, 
                  PositionalBinding=$false,
                  ConfirmImpact='Medium')]
    [Alias()]
    [OutputType([PSOBJECT])]
    Param (
        [Parameter (Mandatory=$False)]
        [string]$BaseConfigPath = (Split-Path $PSScriptRoot -Parent) + "\Base.config"
    )

    [xml]$BaseConfig = Get-Content $BaseConfigPath

    Write-Output "`nTime to generate a configuration for ELHC.  This command just does the basics -- to fine tune, modify ELHC.config in your favorite text editor.`n"
    
    [string]$DepartmentName = Read-Host -Prompt "What is the Department name (usually a city/county)"
    do {
        [string]$DepartmentState = Read-Host -Prompt "What is the Department state (e.g., MN or TX)"
    } while ($DepartmentState -notmatch '^[A-Z]{2,2}$')

    do {
        [string]$DepartmentType = Read-Host -Prompt "What is the Department type (PD, SO, StateDepartment, Federal, Commercial, or Other)"
    } while ($DepartmentType -notin ("PD","SO","StateDepartment","Other", "Federal", "Commercial"))
    
    $BaseConfig.root.DepartmentName = $DepartmentName #Example TX, SO    future #Example
    $BaseConfig.root.DepartmentState = $DepartmentState #TX
    $BaseConfig.root.DepartmentType = $DepartmentType #SO
    
    do {
        [string]$DepartmentSize = Read-Host -Prompt "What is the size of the Department -- small, medium, large, or XL? (s/m/l/xl)"
    } while ($DepartmentSize -notin ('s','m','l','xl'))

    [string]$DepartmentPOC = Read-Host -Prompt "Please provide a name and email/phone for POC for this agency"

    $BaseConfig.root.DepartmentPOC = $DepartmentPOC

    switch ($DepartmentSize) {
        's' {
            $BaseConfig.root.components.standalone.component.jobqueue.alerts.allowedconcats = '50'
            $BaseConfig.root.components.standalone.component.jobqueue.alerts.allowedtrans = '50'
            $BaseConfig.root.components.avdurow.component.storageinfo.alerts.allowedusage = '80'
            $BaseConfig.root.components.standalone.component.bitsstatus.alerts.allowederrorjobs = '10'
            $BaseConfig.root.components.standalone.component.purgeinfo.alerts.alloweddays = '14'
            $BaseConfig.root.components.standalone.component.importinfo.alerts.alloweddays = '7'
            $BaseConfig.root.DepartmentSize = "Small"
        }

        'm' {
            $BaseConfig.root.components.standalone.component.jobqueue.alerts.allowedconcats = '100'
            $BaseConfig.root.components.standalone.component.jobqueue.alerts.allowedtrans = '100'
            $BaseConfig.root.components.avdurow.component.storageinfo.alerts.allowedusage = '90'
            $BaseConfig.root.components.standalone.component.bitsstatus.alerts.allowederrorjobs = '25'
            $BaseConfig.root.components.standalone.component.purgeinfo.alerts.alloweddays = '10'
            $BaseConfig.root.components.standalone.component.importinfo.alerts.alloweddays = '5'
            $BaseConfig.root.DepartmentSize = "Medium"
        }

        'l' {
            $BaseConfig.root.components.standalone.component.jobqueue.alerts.allowedconcats = '250'
            $BaseConfig.root.components.standalone.component.jobqueue.alerts.allowedtrans = '250'
            $BaseConfig.root.components.avdurow.component.storageinfo.alerts.allowedusage = '90'
            $BaseConfig.root.components.standalone.component.bitsstatus.alerts.allowederrorjobs = '100'
            $BaseConfig.root.components.standalone.component.purgeinfo.alerts.alloweddays = '7'
            $BaseConfig.root.components.standalone.component.importinfo.alerts.alloweddays = '3'
            $BaseConfig.root.DepartmentSize = "Large"
        }
        'xl' {
            $BaseConfig.root.components.standalone.component.jobqueue.alerts.allowedconcats = '750'
            $BaseConfig.root.components.standalone.component.jobqueue.alerts.allowedtrans = '750'
            $BaseConfig.root.components.avdurow.component.storageinfo.alerts.allowedusage = '90'
            $BaseConfig.root.components.standalone.component.bitsstatus.alerts.allowederrorjobs = '150'
            $BaseConfig.root.components.standalone.component.purgeinfo.alerts.alloweddays = '5'
            $BaseConfig.root.components.standalone.component.importinfo.alerts.alloweddays = '2'
            $BaseConfig.root.DepartmentSize = "XL"
        }
    }


    $ConnectionStringInstance = Read-Host -Prompt "Where is the SQL instance?  (if this is an all-in-one, press enter)"
    if ($ConnectionStringInstance -eq $null -or $ConnectionStringInstance -eq "") {
        $ConnectionStringInstance = '(local)'
    }
    
    $BaseConfig.root.ConnectionString = "data source=$ConnectionStringInstance;initial catalog=WGEvidenceLibrary;integrated security=true;connection lifetime=30;"
    $BaseConfig.root.ELHCVersion = (Get-InstalledModule ELHealthcheck).Version.ToString()
    
    try {
        $DepartmentID = Get-QueryResult -query "select DepartmentID as ID from dbo.Department" -connectionString $BaseConfig.root.ConnectionString
        $BaseConfig.root.DepartmentID = $DepartmentID.ID.Guid.tostring()
    }
    catch {
        Write-Warning "Could not grab connection string from SQL.  Verify the SQL instance is correct, or specify manually..."
    }
    
    $OldErrorPref = $ErrorActionPreference
            
    $Substation = . {
        $DriveLetters = @('C','D','E')
        foreach ($Letter in $DriveLetters) {
            $Path = "$($Letter):\Program Files\WatchGuard Video\Hosted Services\Services.Host.WinService.exe.config"
            if (test-path $Path) {
                [xml]$HostedConfig = Get-Content -Path $path
                break
            }
            else {
                write-warning "Didn't find anything at $Path..."
            }
        }


        $SSID = ($HostedConfig.configuration.appSettings.add | where {$_.key -eq 'SubstationIdentity'}).value
        if ($SSID -eq $null -or $HostedConfig -eq $null) {
            Write-Warning "Could not find SubstationID in the hosted config.  You'll have to specify this manually."
        }
        else {
            Write-Output "Substation ID : $SSID"
            $BaseConfig.root.SubstationID = $SSID
        }
    }

    $ErrorActionPreference = $OldErrorPref

    $LegacySMB = [Environment]::OSVersion.Version -le (new-object 'Version' 6,2)
    if ($LegacySMB) {
        ($BaseConfig.root.components.avdurow.component | where {$_.name -eq 'Test-SharePermissions'}).enabled = 'FALSE'
        ($BaseConfig.root.components.avdurow.component | where {$_.name -eq 'Test-SharePermissionsLegacy'}).enabled = 'TRUE'
    }

    Write-Output "`n---------------------`n"
    Write-Output "Enough data collection -- please review these values:`n"

    $BaseSettings = $BaseConfig.root.childnodes | where {$_.name -ne 'components'}
    foreach ($Setting in $BaseSettings | where {$_.Name -ne 'CustomerID' -and $_.Name -ne 'SharedKey'}) {
        Write-Output "$($Setting.name) : $($Setting.'#text')"
    }

    $Save = Read-Host "`nDo these values look correct? (y/n)"

    if ($Save -eq 'y') {
        $CurrentModPath = (Split-Path $PSScriptRoot -Parent)
        $BaseModPath = Split-Path $CurrentModPath -Parent
        $ConfigPath = "$BaseModPath\$($BaseConfig.root.ELHCversion)\ELHC.config"
        $BaseConfig.Save($ConfigPath)
        Write-Output "`nConfig file built and saved at $ConfigPath"
        
    }
        else {
        Write-Output "Config file was not saved... try again."
    }

    $XMLPath = (Split-Path $PSScriptRoot -Parent) + "\ELHealthcheck.xml"
    Write-Output "Making scheduled task... enter password for $env:USERNAME when prompted"
    schtasks /Create /XML $XMLPath /TN ELHealthcheck /RU $env:USERNAME /RP
}